@@ -24,7 +24,7 @@ use Spreadsheet::WriteExcel::Formula;
 use vars qw($VERSION @ISA);
 @ISA = qw(Spreadsheet::WriteExcel::BIFFwriter);
 
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 ###############################################################################
 #
@@ -93,11 +93,17 @@ sub new {
 
     $self->{_fit_page}          = 0;
     $self->{_fit_width}         = 1;
-    $self->{_fit_heigth}        = 1;
+    $self->{_fit_height}        = 1;
 
     $self->{_hbreaks}           = [];
     $self->{_vbreaks}           = [];
 
+    $self->{_protect}           = 0;
+    $self->{_password}          = undef;
+
+    $self->{_col_sizes}         = {};
+    $self->{_row_sizes}         = {};
+
     bless $self, $class;
     $self->_initialize();
     return $self;
@@ -153,6 +159,12 @@ sub _close {
     # Prepend the sheet dimensions
     $self->_store_dimensions();
 
+    # Prepend the sheet password
+    $self->_store_password();
+
+    # Prepend the sheet protection
+    $self->_store_protect();
+
     # Prepend the page setup
     $self->_store_setup();
 
@@ -324,6 +336,23 @@ sub set_first_sheet {
 }
 
 
+###############################################################################
+#
+# protect($password)
+#
+# Set the worksheet protection flag to prevent accidental modification and to
+# hide formulas if the locked and hidden format properties have been set.
+#
+sub protect {
+
+    my $self = shift;
+
+    $self->{_protect}   = 1;
+    $self->{_password}  = $self->_encode_password($_[0]) if defined $_[0];
+
+}
+
+
 ###############################################################################
 #
 # set_column($firstcol, $lastcol, $width, $format, $hidden)
@@ -342,6 +371,21 @@ sub set_column {
     }
 
     push @{$self->{_colinfo}}, [ @_ ];
+
+
+    # Store the col sizes for use when calculating image vertices
+    # Take hidden columns into account
+    #
+    return if @_ < 3;# Ensure at least $firstcol, $lastcol and $width
+
+    # Set width to zero if column is hidden
+    my $width = $_[4] ? 0 : $_[2];
+
+    my ($firstcol, $lastcol) = @_;
+
+    foreach my $col ($firstcol .. $lastcol) {
+        $self->{_col_sizes}->{$col} = $width;
+    }
 }
 
 
@@ -723,7 +767,7 @@ sub fit_to_pages {
 
     $self->{_fit_page}      = 1;
     $self->{_fit_width}     = $_[0] || 1;
-    $self->{_fit_heigth}    = $_[1] || 1;
+    $self->{_fit_height}    = $_[1] || 1;
 }
 
 
@@ -757,7 +801,7 @@ sub set_v_pagebreaks {
 
 ###############################################################################
 #
-# write ($row, $col, $token, $format)
+# write($row, $col, $token, $format)
 #
 # Parse $token call appropriate write method. $row and $column are zero
 # indexed. $format is optional.
@@ -775,6 +819,12 @@ sub write {
 
     my $token = $_[2];
 
+
+    # Match an array ref.
+    if (ref $token eq "ARRAY") {
+        return $self->write_row(@_);
+    }
+
     # Match number
     if ($token =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/) {
         return $self->write_number(@_);
@@ -803,6 +853,122 @@ sub write {
 }
 
 
+###############################################################################
+#
+# write_row($row, $col, $array_ref, $format)
+#
+# Write a row of data starting from ($row, $col). Call write_col() if any of
+# the elements of the array ref are in turn array refs. This allows the writing
+# of 1D or 2D arrays of data in one go.
+#
+# Returns: the first encountered error value or zero for no errors
+#
+sub write_row {
+
+    my $self    = shift;
+
+
+    # Check for a cell reference in A1 notation and substitute row and column
+    if ($_[0] =~ /^\D/) {
+        @_ = $self->_substitute_cellref(@_);
+    }
+
+    # Catch non array refs passed by user.
+    if (ref $_[2] ne 'ARRAY') {
+        croak "Not an array ref in call to write_row()$!";
+    }
+
+    my $row     = shift;
+    my $col     = shift;
+    my $tokens  = shift;
+    my @options = @_;
+    my $error   = 0;
+    my $ret;
+
+    foreach my $token (@$tokens) {
+        # Deal with undef values. If a format has been specified we write a
+        # formatted blank cell. Otherwise we ignore the undef.
+        #
+        if (not defined $token) {
+            if (not @options) {
+                $col++;
+                next;
+            }
+            $token = '' unless defined $token;
+        }
+
+        # Check for nested arrays
+        if (ref $token eq "ARRAY") {
+            $ret = $self->write_col($row, $col, $token, @options);
+        } else {
+            $ret = $self->write    ($row, $col, $token, @options);
+        }
+
+        # Return only the first error encountered, if any.
+        $error = $ret if ($ret and not $error);
+        $col++;
+    }
+
+    return $error;
+}
+
+
+###############################################################################
+#
+# write_col($row, $col, $array_ref, $format)
+#
+# Write a column of data starting from ($row, $col). Call write_row() if any of
+# the elements of the array ref are in turn array refs. This allows the writing
+# of 1D or 2D arrays of data in one go.
+#
+# Returns: the first encountered error value or zero for no errors
+#
+sub write_col {
+
+    my $self    = shift;
+
+
+    # Check for a cell reference in A1 notation and substitute row and column
+    if ($_[0] =~ /^\D/) {
+        @_ = $self->_substitute_cellref(@_);
+    }
+
+    # Catch non array refs passed by user.
+    if (ref $_[2] ne 'ARRAY') {
+        croak "Not an array ref in call to write_row()$!";
+    }
+
+    my $row     = shift;
+    my $col     = shift;
+    my $tokens  = shift;
+    my @options = @_;
+    my $error   = 0;
+    my $ret;
+
+    foreach my $token (@$tokens) {
+        # Deal with undef values. If a format has been specified we write a
+        # formatted blank cell. Otherwise we ignore the undef.
+        #
+        if (not defined $token) {
+            if (not @options) {
+                $row++;
+                next;
+            }
+            $token = '' unless defined $token;
+        }
+
+        # write() will deal with any nested arrays
+        $ret = $self->write($row, $col, $token, @options);
+
+        # Return only the first error encountered, if any.
+        $error = $ret if ($ret and not $error);
+        $row++;
+    }
+
+    return $error;
+}
+
+
 ###############################################################################
 #
 # _XF()
@@ -843,6 +1009,9 @@ sub _append {
     if ($self->{_using_tmpfile}) {
         my $data = join('', @_);
 
+        # Add CONTINUE records if necessary
+        $data = $self->_add_continue($data) if length($data) > $self->{_limit};
+
         # Protect print() from -l on the command line.
         local $\ = undef;
 
@@ -903,6 +1072,8 @@ sub _substitute_cellref {
 #
 # Returns: row, column
 #
+# TODO use function in Utility.pm
+#
 sub _cell_to_rowcol {
 
     my $self = shift;
@@ -955,13 +1126,28 @@ sub _sort_pagebreaks {
     @array    = sort {$a <=> $b} keys %hash; # Numerical sort
     shift @array if $array[0] == 0;          # Remove zero
 
-    # Limit the max data size until CONTINUE is implemented
-    splice(@array, 1039) if (@array > 1039);
+    # 1000 vertical pagebreaks appears to be an internal Excel 5 limit.
+    # It is slightly higher in Excel 97/200, approx 1026
+    splice(@array, 1000) if (@array > 1000);
 
     return @array
 }
 
 
+###############################################################################
+#
+# _encode_password($password)
+#
+# TODO. This is only a placeholder. Please be patient.
+#
+sub _encode_password {
+
+    my $self = shift;
+
+    return 0;
+}
+
+
 ###############################################################################
 ###############################################################################
 #
@@ -1223,7 +1409,7 @@ sub write_url {
     }
 
     if (@_ < 3) { return -1 }                    # Check the number of args
-    
+
     # Reverse the order of $string and $format if necessary.
     ($_[3], $_[4]) = ($_[4], $_[3]) if (ref $_[3]);
 
@@ -1308,6 +1494,12 @@ sub set_row {
                                     $irwMac,$reserved, $grbit, $ixfe);
 
     $self->_append($header, $data);
+
+    # Store the row sizes for use when calculating image vertices
+    #
+    return if @_ < 2;# Ensure at least $row and $height
+
+    $self->{_row_sizes}->{$_[0]} = $_[1];
 }
 
 
@@ -1633,7 +1825,7 @@ sub _store_setup {
     my $iScale       = 0x64;                    # Scaling factor
     my $iPageStart   = 0x01;                    # Starting page number
     my $iFitWidth    = $self->{_fit_width};     # Fit to number of pages wide
-    my $iFitHeight   = $self->{_fit_heigth};    # Fit to number of pages high
+    my $iFitHeight   = $self->{_fit_height};    # Fit to number of pages high
     my $grbit        = 0x00;                    # Option flags
     my $iRes         = 0x0258;                  # Print resolution
     my $iVRes        = 0x0258;                  # Vertical print resolution
@@ -2059,7 +2251,6 @@ sub _store_hbreak {
 #
 sub _store_vbreak {
 
-
     my $self    = shift;
 
     # Return if the user hasn't specified pagebreaks
@@ -2085,6 +2276,454 @@ sub _store_vbreak {
 }
 
 
+###############################################################################
+#
+# _store_protect()
+#
+# Set the Biff PROTECT record to indicate that the worksheet is protected.
+#
+sub _store_protect {
+
+    my $self        = shift;
+
+    # Exit unless sheet protection has been specified
+    return unless $self->{_protect};
+
+    my $record      = 0x0012;               # Record identifier
+    my $length      = 0x0002;               # Bytes to follow
+
+    my $fLock       = $self->{_protect};    # Worksheet is protected
+
+    my $header      = pack("vv", $record, $length);
+    my $data        = pack("v",  $fLock);
+
+    $self->_prepend($header, $data);
+}
+
+
+###############################################################################
+#
+# _store_password()
+#
+# Write the worksheet PASSWORD record.
+#
+sub _store_password {
+
+    my $self        = shift;
+
+    # Exit unless sheet protection and password have been specified
+    return unless $self->{_protect} and defined $self->{_password};
+
+    my $record      = 0x0013;               # Record identifier
+    my $length      = 0x0002;               # Bytes to follow
+
+    my $wPassword   = $self->{_password};   # Encoded password
+
+    my $header      = pack("vv", $record, $length);
+    my $data        = pack("v",  $wPassword);
+
+    $self->_prepend($header, $data);
+}
+
+
+###############################################################################
+#
+# insert_bitmap($row, $col, $filename, $x, $y, $scale_x, $scale_y)
+#
+# Insert a 24bit bitmap image in a worksheet. The main record required is
+# IMDATA but it must be proceeded by a OBJ record to define its position.
+#
+sub insert_bitmap {
+
+    my $self        = shift;
+
+    # Check for a cell reference in A1 notation and substitute row and column
+    if ($_[0] =~ /^\D/) {
+        @_ = $self->_substitute_cellref(@_);
+    }
+
+    my $row         = $_[0];
+    my $col         = $_[1];
+    my $bitmap      = $_[2];
+    my $x           = $_[3] || 0;
+    my $y           = $_[4] || 0;
+    my $scale_x     = $_[5] || 1;
+    my $scale_y     = $_[6] || 1;
+
+    my ($width, $height, $size, $data) = $self-> _process_bitmap($bitmap);
+
+    # Scale the frame of the image.
+    $width  *= $scale_x;
+    $height *= $scale_y;
+
+    # Calculate the vertices of the image and write the OBJ record
+    $self->_position_image($col, $row, $x, $y, $width, $height);
+
+
+    # Write the IMDATA record to store the bitmap data
+    my $record      = 0x007f;
+    my $length      = 8 + $size;
+    my $cf          = 0x09;
+    my $env         = 0x01;
+    my $lcb         = $size;
+
+    my $header      = pack("vvvvV", $record, $length, $cf, $env, $lcb);
+
+    $self->_append($header, $data);
+}
+
+
+###############################################################################
+#
+#  _position_image()
+#
+# Calculate the vertices that define the position of the image as required by
+# the OBJ record.
+#
+#         +------------+------------+
+#         |     A      |      B     |
+#   +-----+------------+------------+
+#   |     |(x1,y1)     |            |
+#   |  1  |(A1)._______|______      |
+#   |     |    |              |     |
+#   |     |    |              |     |
+#   +-----+----|    BITMAP    |-----+
+#   |     |    |              |     |
+#   |  2  |    |______________.     |
+#   |     |            |        (B2)|
+#   |     |            |     (x2,y2)|
+#   +---- +------------+------------+
+#
+# Example of a bitmap that covers some of the area from cell A1 to cell B2.
+#
+# Based on the width and height of the bitmap we need to calculate 8 vars:
+#     $col_start, $row_start, $col_end, $row_end, $x1, $y1, $x2, $y2.
+# The width and height of the cells are also variable and have to be taken into
+# account.
+# The values of $col_start and $row_start are passed in from the calling
+# function. The values of $col_end and $row_end are calculated by subtracting
+# the width and height of the bitmap from the width and height of the
+# underlying cells.
+# The vertices are expressed as a percentage of the underlying cell width as
+# follows (rhs values are in pixels):
+#
+#       x1 = X / W *1024
+#       y1 = Y / H *256
+#       x2 = (X-1) / W *1024
+#       y2 = (Y-1) / H *256
+#
+#       Where:  X is distance from the left side of the underlying cell
+#               Y is distance from the top of the underlying cell
+#               W is the width of the cell
+#               H is the height of the cell
+#
+# Note: the SDK incorrectly states that the height should be expressed as a
+# percentage of 1024.
+#
+sub _position_image {
+
+    my $self = shift;
+
+    my $col_start;  # Col containing upper left corner of object
+    my $x1;         # Distance to left side of object
+
+    my $row_start;  # Row containing top left corner of object
+    my $y1;         # Distance to top of object
+
+    my $col_end;    # Col containing lower right corner of object
+    my $x2;         # Distance to right side of object
+
+    my $row_end;    # Row containing bottom right corner of object
+    my $y2;         # Distance to bottom of object
+
+    my $width;      # Width of image frame
+    my $height;     # Height of image frame
+
+    ($col_start, $row_start, $x1, $y1, $width, $height) = @_;
+
+    # Initialise end cell to the same as the start cell
+    $col_end    = $col_start;
+    $row_end    = $row_start;
+
+    # Zero the specified offset if greater than the cell dimensions
+    $x1         = 0 if $x1 >= $self->_size_col($col_start);
+    $y1         = 0 if $y1 >= $self->_size_row($row_start);
+
+    $width      = $width  + $x1 -1;
+    $height     = $height + $y1 -1;
+
+    # Subtract the underlying cell widths to find the end cell of the image
+    while ($width >= $self->_size_col($col_end)) {
+        $width -= $self->_size_col($col_end);
+        $col_end++;
+    }
+
+    # Subtract the underlying cell heights to find the end cell of the image
+    while ($height >= $self->_size_row($row_end)) {
+        $height -= $self->_size_row($row_end);
+        $row_end++;
+    }
+
+    # Bitmap isn't allowed to start or finish in a hidden cell, i.e. a cell
+    # with zero eight or width.
+    #
+    return if $self->_size_col($col_start) == 0;
+    return if $self->_size_col($col_end)   == 0;
+    return if $self->_size_row($row_start) == 0;
+    return if $self->_size_row($row_end)   == 0;
+
+    # Convert the pixel values to the percentage value expected by Excel
+    $x1 = $x1     / $self->_size_col($col_start)   * 1024;
+    $y1 = $y1     / $self->_size_row($row_start)   *  256;
+    $x2 = $width  / $self->_size_col($col_end)     * 1024;
+    $y2 = $height / $self->_size_row($row_end)     *  256;
+
+    $self->_store_obj_picture(  $col_start, $x1,
+                                $row_start, $y1,
+                                $col_end, $x2,
+                                $row_end, $y2
+                             );
+}
+
+
+###############################################################################
+#
+# _size_col($col)
+#
+#
+# Convert the width of a cell from user's units to pixels. By interpolation
+# the relationship is: y = 7x +5. If the width hasn't been set by the user we
+# use the default value. If the col is hidden we use a value of zero.
+#
+sub _size_col {
+
+    my $self = shift;
+    my $col  = $_[0];
+
+    # Look up the cell value to see if it has been changed
+    if (exists $self->{_col_sizes}->{$col}) {
+        if ($self->{_col_sizes}->{$col} == 0) {
+            return 0;
+        }
+        else {
+            return int (7 * $self->{_col_sizes}->{$col} + 5);
+        }
+    }
+    else {
+        return 64;
+    }
+}
+
+
+###############################################################################
+#
+# _size_row($row)
+#
+# Convert the height of a cell from user's units to pixels. By interpolation
+# the relationship is: y = 4/3x. If the height hasn't been set by the user we
+# use the default value. If the row is hidden we use a value of zero. (Not
+# possible to hide row yet).
+#
+sub _size_row {
+
+    my $self = shift;
+    my $row  = $_[0];
+
+    # Look up the cell value to see if it has been changed
+    if (exists $self->{_row_sizes}->{$row}) {
+        if ($self->{_row_sizes}->{$row} == 0) {
+            return 0;
+        }
+        else {
+            return int (4/3 * $self->{_row_sizes}->{$row});
+        }
+    }
+    else {
+        return 17;
+    }
+}
+
+
+###############################################################################
+#
+# _store_obj_picture(   $col_start, $x1,
+#                       $row_start, $y1,
+#                       $col_end,   $x2,
+#                       $row_end,   $y2 )
+#
+# Store the OBJ record that precedes an IMDATA record. This could be generalise
+# to support other Excel objects.
+#
+sub _store_obj_picture {
+
+    my $self        = shift;
+
+    my $record      = 0x005d;   # Record identifier
+    my $length      = 0x003c;   # Bytes to follow
+
+    my $cObj        = 0x0001;   # Count of objects in file (set to 1)
+    my $OT          = 0x0008;   # Object type. 8 = Picture
+    my $id          = 0x0001;   # Object ID
+    my $grbit       = 0x0614;   # Option flags
+
+    my $colL        = $_[0];    # Col containing upper left corner of object
+    my $dxL         = $_[1];    # Distance from left side of cell
+
+    my $rwT         = $_[2];    # Row containing top left corner of object
+    my $dyT         = $_[3];    # Distance from top of cell
+
+    my $colR        = $_[4];    # Col containing lower right corner of object
+    my $dxR         = $_[5];    # Distance from right of cell
+
+    my $rwB         = $_[6];    # Row containing bottom right corner of object
+    my $dyB         = $_[7];    # Distance from bottom of cell
+
+    my $cbMacro     = 0x0000;   # Length of FMLA structure
+    my $Reserved1   = 0x0000;   # Reserved
+    my $Reserved2   = 0x0000;   # Reserved
+
+    my $icvBack     = 0x09;     # Background colour
+    my $icvFore     = 0x09;     # Foreground colour
+    my $fls         = 0x00;     # Fill pattern
+    my $fAuto       = 0x00;     # Automatic fill
+    my $icv         = 0x08;     # Line colour
+    my $lns         = 0xff;     # Line style
+    my $lnw         = 0x01;     # Line weight
+    my $fAutoB      = 0x00;     # Automatic border
+    my $frs         = 0x0000;   # Frame style
+    my $cf          = 0x0009;   # Image format, 9 = bitmap
+    my $Reserved3   = 0x0000;   # Reserved
+    my $cbPictFmla  = 0x0000;   # Length of FMLA structure
+    my $Reserved4   = 0x0000;   # Reserved
+    my $grbit2      = 0x0001;   # Option flags
+    my $Reserved5   = 0x0000;   # Reserved
+
+
+    my $header      = pack("vv", $record, $length);
+    my $data        = pack("V",  $cObj);
+       $data       .= pack("v",  $OT);
+       $data       .= pack("v",  $id);
+       $data       .= pack("v",  $grbit);
+       $data       .= pack("v",  $colL);
+       $data       .= pack("v",  $dxL);
+       $data       .= pack("v",  $rwT);
+       $data       .= pack("v",  $dyT);
+       $data       .= pack("v",  $colR);
+       $data       .= pack("v",  $dxR);
+       $data       .= pack("v",  $rwB);
+       $data       .= pack("v",  $dyB);
+       $data       .= pack("v",  $cbMacro);
+       $data       .= pack("V",  $Reserved1);
+       $data       .= pack("v",  $Reserved2);
+       $data       .= pack("C",  $icvBack);
+       $data       .= pack("C",  $icvFore);
+       $data       .= pack("C",  $fls);
+       $data       .= pack("C",  $fAuto);
+       $data       .= pack("C",  $icv);
+       $data       .= pack("C",  $lns);
+       $data       .= pack("C",  $lnw);
+       $data       .= pack("C",  $fAutoB);
+       $data       .= pack("v",  $frs);
+       $data       .= pack("V",  $cf);
+       $data       .= pack("v",  $Reserved3);
+       $data       .= pack("v",  $cbPictFmla);
+       $data       .= pack("v",  $Reserved4);
+       $data       .= pack("v",  $grbit2);
+       $data       .= pack("V",  $Reserved5);
+
+    $self->_append($header, $data);
+}
+
+
+###############################################################################
+#
+# _process_bitmap()
+#
+# Convert a 24 bit bitmap into the modified internal format used by Windows.
+# This is described in BITMAPCOREHEADER and BITMAPCOREINFO structures in the
+# MSDN library.
+#
+sub _process_bitmap {
+
+    my $self   = shift;
+    my $bitmap = shift;
+
+    # Open file and binmode the data in case the platform needs it.
+    open    BMP, $bitmap  or croak "Couldn't import $bitmap: $!\n";
+    binmode BMP;
+
+
+    # Slurp the file into a string.
+    my $data = do {local $/; <BMP>};
+
+
+    # Check that the file is big enough to be a bitmap.
+    if (length $data <= 0x36) {
+        croak "$bitmap doesn't contain enough data.\n";
+    }
+
+
+    # The first 2 bytes are used to identify the bitmap.
+    if (unpack("A2", $data) ne "BM") {
+        croak "$bitmap doesn't appear to to be a valid bitmap image.\n";
+    }
+
+
+    # Remove bitmap data: ID.
+    $data = substr $data, 2;
+
+
+    # Read and remove the bitmap size. This is more reliable than reading
+    # the data size at offset 0x22.
+    #
+    my $size   =  unpack "V", substr $data, 0, 4, "";
+       $size  -=  0x36;   # Subtract size of bitmap header.
+       $size  +=  0x0C;   # Add size of BIFF header.
+
+
+    # Remove bitmap data: reserved, offset, header length.
+    $data = substr $data, 12;
+
+
+    # Read and remove the bitmap width and height. Verify the sizes.
+    my ($width, $height) = unpack "V2", substr $data, 0, 8, "";
+
+    if ($width > 0xFFFF) {
+        croak "$bitmap: largest image width supported is 65k.\n";
+    }
+
+    if ($height > 0xFFFF) {
+        croak "$bitmap: largest image height supported is 65k.\n";
+    }
+
+    # Read and remove the bitmap planes and bpp data. Verify them.
+    my ($planes, $bitcount) = unpack "v2", substr $data, 0, 4, "";
+
+    if ($bitcount != 24) {
+        croak "$bitmap isn't a 24bit true color bitmap.\n";
+    }
+
+    if ($planes != 1) {
+        croak "$bitmap: only 1 plane supported in bitmap image.\n";
+    }
+
+
+    # Read and remove the bitmap compression. Verify compression.
+    my $compression = unpack "V", substr $data, 0, 4, "";
+
+    if ($compression != 0) {
+        croak "$bitmap: compression not supported in bitmap image.\n";
+    }
+
+    # Remove bitmap data: data size, hres, vres, colours, imp. colours.
+    $data = substr $data, 20;
+
+    # Add the BITMAPCOREHEADER data
+    my $header  = pack("Vvvvv", 0x000c, $width, $height, 0x01, 0x18);
+    $data       = $header . $data;
+
+    return ($width, $height, $size, $data);
+}
 
 
 1;
